package com.bgy.sms.channel.bgy.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.bgy.sms.channel.bgy.config.BgySMSConfig;
import com.bgy.sms.channel.bgy.dto.CLBizResponse;
import com.bgy.sms.channel.bgy.dto.TemplateNotify;
import com.bgy.sms.channel.bgy.service.BgySmsService;
import com.bgy.sms.channel.bgy.utils.SendSmsUtil;
import com.bgy.sms.channel.dmHub.service.DmHubService;
import com.bgy.sms.channel.dto.BGYResponse;
import com.bgy.sms.config.ResponseCode;
import com.bgy.sms.repository.domain.SmsTemplateInfo;
import com.bgy.sms.service.SmsTemplateService;
import com.bgy.sms.service.XiaoShuService;
import com.bgy.util.HttpUtil;
import com.bgy.util.Md5Util;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@SuppressWarnings({"DuplicatedCode", "Duplicates"})
@Service
public class BgySmsServiceImpl implements BgySmsService {

    private static final Logger log = LoggerFactory.getLogger(BgySmsServiceImpl.class);
    private static final String SUCCESS = "success";
    private static final String Fail = "Fail";
    private static final String OK = "Fail";
    private static final String sendSuccessCode = "0";  // 发送响应成功字段
    private static final String createTemplateStatus = "status"; // 响应状态字段
    private static final String retMsg = "msg";                 //  响应描述
    private static final String createTemplateData = "data";    //  创建模板响应数据

    private static final String REJECTED = "rejected";    //  创建模板响应数据
    private static final String APPROVED = "approved";    //  创建模板响应数据

    private static final String notificationId = "49"; // 碧桂园通知类短信ID
    private static final String marketingId = "52";    // 碧桂园营销类短信ID

    @Autowired
    private SmsTemplateService templateService;
    @Autowired
    private DmHubService dmHubService;

    @Autowired
    private SmsTemplateService smsTemplateService;

    @Autowired
    private XiaoShuService xiaoShuService;

    @Override
    public CLBizResponse sendSms(String mobile, String content, String areaId, String subModule, String api) throws Exception {
        log.info("进入碧桂园短信发送接口");

        String appId = BgySMSConfig.appId;
        String securityCode = BgySMSConfig.securityCode;
        String url = BgySMSConfig.url;
        if (api == null) {
            api = BgySMSConfig.api;
        }
        Map<String, String> requestParams = new HashMap<>();
        requestParams.put("api", api);
        requestParams.put("appId", appId);
        requestParams.put("security", Md5Util.encrypt(appId + securityCode.toUpperCase()));
        requestParams.put("areaId", areaId);
        requestParams.put("subModule", subModule);
        requestParams.put("content", content);
        requestParams.put("mobile", mobile);
        log.info("碧桂园短信接口参数：{}", JSONObject.toJSONString(requestParams));
        long start = System.currentTimeMillis();
        String retStr = SendSmsUtil.sendSmsByPost(url, JSONObject.toJSONString(requestParams));
        log.info("碧桂园短信接口返回信息：{}", retStr);
        long end = System.currentTimeMillis();
        log.info("**********碧桂园短信接口耗时*******：{}ms", end - start);
        if (retStr == null) {
            return new CLBizResponse(ResponseCode.UPSTREAM_BLANK);
        }
        JSONObject retJson = JSONObject.parseObject(retStr);
        String err = retJson.getString("err");
        String retPack = retJson.getString("package");
        String retCode = retJson.getString("ret");
        if (!sendSuccessCode.equals(retCode)) {
            if (err.contains("成功")) {
                return new CLBizResponse(ResponseCode.SUCCESS);
            }
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), StringUtils.isNotBlank(err) ? "碧桂园短信发送接口返回：" + err : "碧桂园短信发送接口返回空");
        } else {
            return new CLBizResponse(ResponseCode.SUCCESS);
        }
    }

    public static void main(String[] args) {
        String encrypt = Md5Util.encrypt("1023" + "f80a1c5c-8f90-401a-98a1-221af5db0a9b".toUpperCase());
        System.out.println(encrypt);
    }

    @Override
    public CLBizResponse sendBatchSms(List<JSONObject> data, String areaId, String subModule, String api, String templateId) throws Exception {
        log.info("进入碧桂园短信发送接口");
        String appId = BgySMSConfig.appId;
        String securityCode = BgySMSConfig.securityCode;
        String url = BgySMSConfig.url;
        Map<String, Object> requestParams = new HashMap<>();
        requestParams.put("api", api);
        requestParams.put("appId", appId);
        requestParams.put("security", Md5Util.encrypt(appId + securityCode.toUpperCase()));
        requestParams.put("areaId", areaId);
        requestParams.put("subModule", subModule);
        requestParams.put("templateId", templateId);
        requestParams.put("data", data);
        log.info("碧桂园短信接口参数：{}", JSONObject.toJSONString(requestParams));
        long start = System.currentTimeMillis();
        String reqStr = JSONObject.toJSONString(requestParams);
        String retStr = SendSmsUtil.sendSmsByPost(url, reqStr);
        log.info("碧桂园短信接口返回信息：{}", retStr);
        long end = System.currentTimeMillis();
        log.info("**********碧桂园短信接口耗时*******：{}ms", end - start);
        if (retStr == null) {
            return new CLBizResponse(ResponseCode.UPSTREAM_BLANK);
        }
        JSONObject retJson = JSONObject.parseObject(retStr);
        String err = retJson.getString("err");
        String retCode = retJson.getString("ret");
        String retPack = retJson.getString("package");
        if (!sendSuccessCode.equals(retCode)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), StringUtils.isNotBlank(err) ? "碧桂园短信发送接口返回：" + err : "碧桂园短信发送接口返回空");
        } else {
            return new CLBizResponse(ResponseCode.SUCCESS);
        }
    }

    /**
     * 处理短信模板的通知
     *
     * @param templateNotify
     * @return
     */
    @Override
    public BGYResponse asyncNotify(TemplateNotify templateNotify) {
        log.info("碧桂园短信模板审核通知接口入参：{}", templateNotify);
        //模板流水号
        String templateRecordId = templateNotify.getData();
        String status = templateNotify.getAuditStatus();
        String auditReason = templateNotify.getAuditReason();
        SmsTemplateInfo smsTemplateInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>().eq("template_record_id", templateRecordId));
        if (smsTemplateInfo == null) {
            return new BGYResponse(2, "未找到短信模板");
        }
        SmsTemplateInfo updateInfo = new SmsTemplateInfo();
        updateInfo.setId(smsTemplateInfo.getId());
        String dmhubStatue = null;
        if ("1".equals(status)) {
            updateInfo.setStatus("normal");
            dmhubStatue = APPROVED;
            updateInfo.setLastUpdated(new Date());
        } else {
            dmhubStatue = REJECTED;
            updateInfo.setStatus("abnormal");
        }
        updateInfo.setUpRejectMsg(auditReason);
        updateInfo.setLastUpdated(new Date());
        smsTemplateService.updateById(updateInfo);
        xiaoShuService.updateDmhubSmsTempStatus(dmhubStatue, smsTemplateInfo.getDmTemplateId(), auditReason);
        return new BGYResponse();
    }


    @Override
    public CLBizResponse createTemplate(String content, String areaId, String account, String templateType, String pkID) throws Exception {
        log.info("进入碧桂园创建模板接口");
        String appId = BgySMSConfig.appId;
        String securityCode = BgySMSConfig.securityCode;
        Map<String, String> param = new HashMap<>();
        param.put("api", "AddTemplate");
        param.put("appid", appId);
        param.put("security", Md5Util.encrypt(appId + securityCode.toUpperCase()));
        param.put("content", content);
        param.put("areaid", areaId);
        param.put("pkID", pkID);
        param.put("slxtID", account);
        param.put("templateType", templateType);

        log.info("碧桂园创建模板接口参数：{}", JSONObject.toJSONString(param));
        String retStr = HttpUtil.sendPost(BgySMSConfig.url, JSONObject.toJSONString(param));
        log.info("碧桂园创建模板接口返回信息：{}", retStr);
        if (retStr == null) {
            return new CLBizResponse(ResponseCode.UPSTREAM_BLANK);
        }
        JSONObject retJson = JSONObject.parseObject(retStr);
        String err = retJson.getString("err");
        String retPack = retJson.getString("package");
        String retCode = retJson.getString("ret");
        if (!sendSuccessCode.equals(retCode)) {
            if (err.contains("成功")) {
                JSONObject Json = JSONObject.parseObject(retPack);
                String data = Json.getString("data");
                return new CLBizResponse(ResponseCode.SUCCESS.getCode(), data);
            }
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), StringUtils.isNotBlank(err) ? "碧桂园模板创建接口返回：" + err : "碧桂园模板创建接口返回空");
        } else {
            JSONObject Json = JSONObject.parseObject(retPack);
            String data = Json.getString("data");
            return new CLBizResponse(ResponseCode.SUCCESS.getCode(), data);
        }
    }
}
